@@ -30,7 +30,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """
 
-import os, sys, platform
+import os, sys
+from copy import deepcopy
 here = os.path.abspath(os.path.dirname(__file__))
 
 
@@ -64,6 +65,7 @@ from rocketcea.separated_Cf import ambientCf, sepNozzleCf
 #
 # gravitational conversion factor
 GC = 32.174
+DEEP_COPY_L = [None] # will hold deepcopy of CEA_Obj (fixes mass/mole fraction calls)
 
 _last_called = None # remember the last object to read the datafile
 _NLines_Max_ever = 0 # make sure to overwrite any lines from previous calls
@@ -345,10 +347,12 @@ class CEA_Obj(object):
                 if fp:
                     fp.close()
 
+        DEEP_COPY_L[0] = deepcopy( self )
 
     def setupCards(self, Pc=100.0, MR=1.0, eps=40.0, subar=None, PcOvPe=None, frozen=0,
                    ERphi=None, ERr=None, frozenAtThroat=0, short_output=0,
-                   show_transport=0, pc_units='psia', output='calories'):
+                   show_transport=0, pc_units='psia', output='calories',
+                   show_mass_frac=False):
         '''
         Set up card deck and call CEA FORTRAN code.::
 
@@ -361,6 +365,9 @@ class CEA_Obj(object):
         '''
 
         global _last_called, _NLines_Max_ever
+        
+        self = DEEP_COPY_L[0]
+
 
         N = 1
         for line in self.cea_deck:
@@ -465,6 +472,8 @@ class CEA_Obj(object):
                     line += " short "
                 if show_transport:
                     line += " transport "
+                if show_mass_frac:
+                    line += " massf "
                     
             set_py_cea_line(N,line)
             N += 1
@@ -532,12 +541,12 @@ class CEA_Obj(object):
             py_cea.trpts.prfro[i] = 0.0  # frozen prandtl number
         
         # set species concentrations to 0.0 prior to calcs
-        for k,p in enumerate(py_cea.cdata.prod):
-            for i in range(3):
-                py_cea.comp.en[k-1,i] = 0.0
-                py_cea.therm.mw[k-1] = 0.0
-            if k>50:
-                break
+        #for k,p in enumerate(py_cea.cdata.prod):
+        #    for i in range(3):
+        #        py_cea.comp.en[k-1,i] = 0.0
+        #        py_cea.therm.mw[k-1] = 0.0
+        #    if k>50:
+        #        break
             
             
         #print( 'calling py_cea with pathPrefix and myfile=' )
@@ -546,7 +555,7 @@ class CEA_Obj(object):
 
     def get_full_cea_output(self, Pc=100.0, MR=1.0, eps=40.0, subar=None, PcOvPe=None,
                             frozen=0, frozenAtThroat=0, short_output=0, show_transport=1,
-                            pc_units='psia', output='calories'):
+                            pc_units='psia', output='calories', show_mass_frac=False):
         """
         Get the full output file created by CEA. Return as a string.
         #: pc_units = 'psia', 'bar', 'atm', 'mmh'(mm of mercury)
@@ -562,7 +571,7 @@ class CEA_Obj(object):
                          frozen=frozen, frozenAtThroat=frozenAtThroat, 
                          short_output=short_output,
                          show_transport=show_transport, pc_units=pc_units,
-                         output=output)
+                         output=output, show_mass_frac=show_mass_frac)
 
         self.makeOutput = save_flag # restore makeOutput
 
@@ -570,7 +579,7 @@ class CEA_Obj(object):
 
 
     def __call__(self, Pc=100.0, MR=1.0, eps=40.0):
-        """Returns IspVac if CEA_Obj is simply called like a function."""
+        """Returns IspVac(sec) if CEA_Obj is simply called like a function."""
         if self.useFastLookup:
             isp =  self.fastModule.get_isp(Pc, eps, MR)
             return isp
@@ -580,7 +589,7 @@ class CEA_Obj(object):
     def get_IvacCstrTc(self, Pc=100.0, MR=1.0, eps=40.0):
         """::
 
-        #: Return the tuple (IspVac, Cstar, Tcomb).
+        #: Return the tuple (IspVac, Cstar, Tcomb)in(sec, ft/sec, degR)
         #: MR is only used for ox/fuel combos.
         """
         if self.useFastLookup:
@@ -624,7 +633,7 @@ class CEA_Obj(object):
     def getFrozen_IvacCstrTc(self, Pc=100.0, MR=1.0, eps=40.0, frozenAtThroat=0):
         """::
 
-        #: Return the tuple (IspFrozen, Cstar, Tcomb).
+        #: Return the tuple (IspFrozen, Cstar, Tcomb)in(sec, ft/sec, degR)
         #: MR is only used for ox/fuel combos.
         #: frozenAtThroat flag, 0=frozen in chamber, 1=frozen at throat
         """
@@ -639,7 +648,7 @@ class CEA_Obj(object):
     def get_IvacCstrTc_exitMwGam(self, Pc=100.0, MR=1.0, eps=40.0):
         """::
 
-        #: return the tuple (IspVac, Cstar, Tcomb, mw, gam)
+        #: return the tuple (IspVac, Cstar, Tcomb, mw, gam)in(sec, ft/sec, degR, lbm/lbmole, -)
         #: mw and gam apply to nozzle exit.
         #: MR is only used for ox/fuel combos.
         """
@@ -658,7 +667,7 @@ class CEA_Obj(object):
     def get_IvacCstrTc_ChmMwGam(self, Pc=100.0, MR=1.0, eps=40.0):
         """::
 
-        #: return the tuple (IspVac, Cstar, Tcomb, mw, gam)
+        #: return the tuple (IspVac, Cstar, Tcomb, mw, gam)in(sec, ft/sec, degR, lbm/lbmole, -)
         #: mw and gam apply to chamber.
         #: MR is only used for ox/fuel combos.
         """
@@ -676,7 +685,7 @@ class CEA_Obj(object):
     def get_IvacCstrTc_ThtMwGam(self, Pc=100.0, MR=1.0, eps=40.0):
         """::
 
-        #: return the tuple (IspVac, Cstar, Tcomb, mw, gam)
+        #: return the tuple (IspVac, Cstar, Tcomb, mw, gam)in(sec, ft/sec, degR, lbm/lbmole, -)
         #: mw and gam apply to throat.
         #: MR is only used for ox/fuel combos.
         """
@@ -694,7 +703,7 @@ class CEA_Obj(object):
     def get_Isp(self, Pc=100.0, MR=1.0, eps=40.0):
         """::
 
-        #: return IspVac.
+        #: return IspVac (sec)
         #: MR is only used for ox/fuel combos.
         """
         if self.useFastLookup:
@@ -721,7 +730,7 @@ class CEA_Obj(object):
     def get_Cstar(self, Pc=100.0, MR=1.0):
         """::
 
-        #: return Cstar.
+        #: return Cstar (ft/sec)
         #: MR is only used for ox/fuel combos.
         """
         if self.useFastLookup:
@@ -746,7 +755,7 @@ class CEA_Obj(object):
     def get_Tcomb(self, Pc=100.0, MR=1.0):
         """::
 
-        #: return Tcomb.
+        #: return Tcomb (degR)
         #: MR is only used for ox/fuel combos.
         """
         if self.useFastLookup:
@@ -821,7 +830,7 @@ class CEA_Obj(object):
     def get_Temperatures(self, Pc=100.0, MR=1.0,eps=40.0, frozen=0, frozenAtThroat=0):
         """::
 
-        #: Return a list of temperatures at the chamber, throat and exit.
+        #: Return a list of temperatures at the chamber, throat and exit (degR)
         #: (Note frozen flag determins whether Texit is equilibrium or Frozen temperature)
         #: MR is only used for ox/fuel combos.
         #: frozen flag (0=equilibrium, 1=frozen)
@@ -838,7 +847,7 @@ class CEA_Obj(object):
     def get_SonicVelocities(self, Pc=100.0, MR=1.0,eps=40.0):
         """::
 
-        #: Return a list of sonic velocities at the chamber, throat and exit.
+        #: Return a list of sonic velocities at the chamber, throat and exit (ft/sec)
         #: MR is only used for ox/fuel combos.
         """
         self.setupCards( Pc=Pc, MR=MR, eps=eps)
@@ -849,7 +858,7 @@ class CEA_Obj(object):
     def get_Chamber_SonicVel(self, Pc=100.0, MR=1.0, eps=40.0):
         """::
 
-        #: Return the sonic velocity in the chamber.
+        #: Return the sonic velocity in the chamber (ft/sec)
         #: MR is only used for ox/fuel combos.
         """
         sonicList = self.get_SonicVelocities( Pc=Pc, MR=MR, eps=eps)
@@ -859,7 +868,7 @@ class CEA_Obj(object):
     def get_Enthalpies(self, Pc=100.0, MR=1.0,eps=40.0):
         """::
 
-        #: Return a list of enthalpies at the chamber, throat and exit in BTU/lbm
+        #: Return a list of enthalpies at the chamber, throat and exit (BTU/lbm)
         #: MR is only used for ox/fuel combos.
         """
         self.setupCards( Pc=Pc, MR=MR, eps=eps)
@@ -882,6 +891,11 @@ class CEA_Obj(object):
         """
         self.setupCards( Pc=Pc, MR=MR, eps=eps, frozen=frozen, frozenAtThroat=frozenAtThroat)
         
+        #s = self.get_full_cea_output( Pc=Pc, MR=MR, eps=eps, subar=None, PcOvPe=None,
+        #                    frozen=frozen, frozenAtThroat=frozenAtThroat, short_output=0, show_transport=1,
+        #                    pc_units='psia', output='calories', show_mass_frac=True)
+        #print('Full Output:',s)
+        
         massFracD = {} # index=species: value=[massfrac_chm, massfrac_tht, massfrac_exit]
         molWtD    = {} # index=species: value=molecular weight
         for k,p in enumerate(py_cea.cdata.prod):
@@ -907,6 +921,7 @@ class CEA_Obj(object):
                             
                     massFracD[p] = mfL
                     molWtD[p] = mw
+                    
         return molWtD, massFracD
 
     def get_SpeciesMoleFractions(self, Pc=100.0, MR=1.0,eps=40.0, frozen=0, frozenAtThroat=0):
@@ -953,7 +968,7 @@ class CEA_Obj(object):
     def get_Chamber_H(self, Pc=100.0, MR=1.0, eps=40.0):
         """::
 
-        #: Return the enthalpy in the chamber in BTU/lbm
+        #: Return the enthalpy in the chamber (BTU/lbm)
         #: MR is only used for ox/fuel combos.
         """
         hList = self.get_Enthalpies( Pc=Pc, MR=MR, eps=eps)
@@ -963,7 +978,7 @@ class CEA_Obj(object):
     def get_Densities(self, Pc=100.0, MR=1.0,eps=40.0):
         """::
 
-        #: Return a list of densities at the chamber, throat and exit.
+        #: Return a list of densities at the chamber, throat and exit(lbm/cuft)
         #: MR is only used for ox/fuel combos.
         """
         self.setupCards( Pc=Pc, MR=MR, eps=eps)
@@ -976,7 +991,7 @@ class CEA_Obj(object):
     def get_Chamber_Density(self, Pc=100.0, MR=1.0, eps=40.0):
         """::
 
-        #: Return the density in the chamber.
+        #: Return the density in the chamber(lbm/cuft)
         #: MR is only used for ox/fuel combos.
         """
         dList = self.get_Densities( Pc=Pc, MR=MR, eps=eps)
@@ -986,7 +1001,7 @@ class CEA_Obj(object):
     def get_HeatCapacities(self, Pc=100.0, MR=1.0,eps=40.0, frozen=0):
         """::
 
-        #: Return a list of heat capacities at the chamber, throat and exit.
+        #: Return a list of heat capacities at the chamber, throat and exit(BTU/lbm degR)
         #: MR is only used for ox/fuel combos.
         #: frozen flag (0=equilibrium, 1=frozen)
         """
@@ -1000,7 +1015,7 @@ class CEA_Obj(object):
     def get_Chamber_Cp(self, Pc=100.0, MR=1.0, eps=40.0, frozen=0):
         """::
 
-        #: Return the heat capacity in the chamber.
+        #: Return the heat capacity in the chamber(BTU/lbm degR)
         #: MR is only used for ox/fuel combos.
         #: frozen flag (0=equilibrium, 1=frozen)
         """
@@ -1010,7 +1025,7 @@ class CEA_Obj(object):
     def get_Throat_Isp(self, Pc=100.0, MR=1.0):
         """::
 
-        #: Return the IspVac for the throat.
+        #: Return the IspVac for the throat(sec).
         #: MR is only used for ox/fuel combos.
         """
         eps=1.0
@@ -1034,7 +1049,7 @@ class CEA_Obj(object):
     def get_Chamber_MolWt_gamma(self, Pc=100.0, MR=1.0, eps=40.0):
         """::
 
-        #: return the tuple (mw, gam) for the chamber
+        #: return the tuple (mw, gam) for the chamber (lbm/lbmole, -)
         #: MR is only used for ox/fuel combos.
         """
         #common /prtout/ cpr,dlvpt,dlvtp,gammas,hsum,ppp,ssum,totn,ttt,vlm,wm,pltout
@@ -1046,7 +1061,7 @@ class CEA_Obj(object):
     def get_Throat_MolWt_gamma(self, Pc=100.0, MR=1.0, eps=40.0):
         """::
 
-        #: return the tuple (mw, gam) for the throat
+        #: return the tuple (mw, gam) for the throat (lbm/lbmole, -)
         #: MR is only used for ox/fuel combos.
         """
         #common /prtout/ cpr,dlvpt,dlvtp,gammas,hsum,ppp,ssum,totn,ttt,vlm,wm,pltout
@@ -1058,7 +1073,7 @@ class CEA_Obj(object):
     def get_exit_MolWt_gamma(self, Pc=100.0, MR=1.0, eps=40.0):
         """::
 
-        #: return the tuple (mw, gam) for the nozzle exit
+        #: return the tuple (mw, gam) for the nozzle exit (lbm/lbmole, -)
         #: MR is only used for ox/fuel combos.
         """
         #common /prtout/ cpr,dlvpt,dlvtp,gammas,hsum,ppp,ssum,totn,ttt,vlm,wm,pltout
